/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////



// OdBrEnums.h -- declaration of enumerated types used in the
//                OdBr B-Rep and traverser classes.


#ifndef ODBRENUMS_H
#define ODBRENUMS_H
#include "../OdError.h"
#include "BrExport.h"

enum OdBrErrorStatus
{ 
  odbrOK                     = eOk,
  odbrWrongObjectType        = eWrongObjectType,
  odbrInvalidObject          = eUnrecoverableErrors,
  odbrUnsuitableTopology     = 0XBC5,
  odbrMissingGeometry        = eDegenerateGeometry,
  odbrInvalidInput           = eInvalidInput,
  odbrDegenerateTopology     = 0XBCC,
  odbrUninitialisedObject    = 0xBCD,
  odbrOutOfMemory            = eOutOfMemory,
  odbrBrepChanged            = 0xBC0,
  odbrNotImplementedYet      = eNotImplementedYet,
  odbrNullObjectId           = eNullObjectId,
  odbrNotApplicable          = eNotApplicable,
  odbrWrongSubentityType     = eWrongSubentityType,
  odbrNullSubentityId        = eInvalidIndex,
  odbrNullObjectPointer      = eUnrecoverableErrors,  
  odbrObjectIdMismatch       = eWrongDatabase,
  odbrTopologyMismatch       = eWrongDatabase,
  odbrUnsuitableGeometry     = eAmbiguousOutput,
  odbrMissingSubentity       = eNotInDatabase,
  odbrAmbiguousOutput        = eAmbiguousOutput,

  odbrUnrecoverableErrors    = eUnrecoverableErrors,
  odbrMissingTopology        = odbrDegenerateTopology,
  odbrWrongDatabase          = eWrongDatabase,
  odbrNotInDatabase          = eNotInDatabase,
  odbrDegenerateGeometry     = eDegenerateGeometry
};

enum BrLoopType{
  odbrLoopUnclassified = 0,
  odbrLoopExterior     = 1,
  odbrLoopInterior     = 2,	
  odbrLoopWinding      = 3,	
};

enum BrShellType{ 
  odbrShellUnclassified = 0,
  odbrShellExterior     = 1,
  odbrShellInterior     = 2
};

enum BrValidationLevel { 
  odbrFullValidation		= 0,
  odbrNoValidation		  = 1
};

/** Description:
  This class is specialization of the OdError class for OdBr errors.  
  Library: TD_Br
  {group:OdBr_Classes}
*/
class ODBR_TOOLKIT_EXPORT OdBrException : public OdError
{
public:
  /**
    Arguments:
    errorStatus (I) Error status.
  */
  OdBrException(OdBrErrorStatus errorStatus);
  /** Description:
    Returns the error status associated with this Exception object.
  */
  OdBrErrorStatus getErrorStatus() const;
};

#define BR_THROW(n) throw OdBrException(n)

#endif // ODBRENUMS_H
