/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////




#ifndef OD_DBWIPEOUTVARIABLES_H
#define OD_DBWIPEOUTVARIABLES_H

#include "TD_PackPush.h"

#include "DbObject.h"


class OdDbWipeoutVariables;
/** Description:
    This template class is a specialization of the OdSmartPtr class for OdDbWipeoutVariables object pointers.
*/
typedef OdSmartPtr<OdDbWipeoutVariables> OdDbWipeoutVariablesPtr;

/** Description:
    Represents the Wipeout Variables object in an OdDbDatabase instance.
    
    Library: TD_Db
  
    {group:OdDb_Classes}
*/
class TOOLKIT_EXPORT OdDbWipeoutVariables : public OdDbObject
{
public:
  ODDB_DECLARE_MEMBERS(OdDbWipeoutVariables);

  OdDbWipeoutVariables();

  virtual OdResult dwgInFields(OdDbDwgFiler* pFiler);

  virtual OdResult dwgOutFields(OdDbDwgFiler* pFiler) const;

  virtual OdResult dxfInFields(OdDbDxfFiler* pFiler);

  virtual OdResult dxfOutFields(OdDbDxfFiler* pFiler) const;

  /** Description:
    Returns the frame display flag for this Variables object (DXF 70).
  */
  bool showFrame() const;

  /** Description:
    Sets the frame display flag for this Variables object (DXF 70).
    Arguments:
    showFrame (I) Controls 
  */
  void setShowFrame(bool showFrame);

  /** Description:
    Opens the Wipeout Variables object in the specified *database*.
    Arguments:
    openMode (I) Mode in which to open the wipeout variables object.
    pDb (I) Pointer to the *database* that contains the wipeout variables object.

    Remarks:
    If the Wipeout Variables object does not exist in the specified *database*
    it is created.
    
    Returns a SmartPointer to the Wipeout Variables object.
  */
  static OdDbWipeoutVariablesPtr openWipeoutVariables(OdDbDatabase* pDb,
    OdDb::OpenMode openMode = OdDb::kForRead);
};

#include "TD_PackPop.h"

#endif //OD_DBWIPEOUTVARIABLES_H

