#ifndef OD_GE_HATCHBUILDER_H
#define OD_GE_HATCHBUILDER_H
#include "GePoint2d.h"
#include "GePoint2dArray.h"
#include "GeKnotVector.h"
#include "GeVoidPointerArray.h"
#include <vector>
#include "GeExport.h"
#include "GeExtents2d.h"

class OdGeHatchBuilderImpl;

class OdGeHatchInterrupt
{
public:
  virtual int interruptCallback() = 0;
};

class GE_TOOLKIT_EXPORT OdGeHatchBuilder 
{
public:
  OdGeHatchBuilder();

  OdGeHatchBuilder( bool isTrimmedHatch, double gapTol = 1.0e-6, double pixelUnitLength = 1.0e-6);

  ~OdGeHatchBuilder();

  void addLine(const OdGePoint2d &rootPt, const OdGeVector2d &direction, void *id);
  void addRay(const OdGePoint2d &rootPt, const OdGeVector2d &direction, void *id);
  void addLineSeg(const OdGePoint2d &startPt, const OdGePoint2d &endPt, void* id);
  void addCirArc(const OdGePoint2d &statPt, const OdGePoint2d &midPt, const OdGePoint2d &endPt, void* id);
  void addCircle(const OdGePoint2d &center, const double radius, void* id);
  void addEllipArc(const OdGePoint2d &center, const OdGeVector2d &majorAxis, const OdGeVector2d &minorAxis, const double majorRadius, const double minorRadius, 
    double startAngle, double endAngle, void* id);
  void addPolyline(const OdGePoint2dArray &pnts, const OdGeDoubleArray &bugles, void *id);
  void addNurb(int degree, bool rational, bool closed, bool periodic, const OdGePoint2dArray &pnts, const OdGeDoubleArray &weights, 
    const OdGeKnotVector &knots, void *id);

  // Build all the loops and regions
  bool evaluateHatch();

  bool beginEvaluateHatch();
  bool addLoops(const std::vector<OdGeVoidPointerArray> &boundary,const std::vector<OdGeVoidPointerArray> &ids);
  bool endEvaluateHatch();

  void setInterruptCallback(OdGeHatchInterrupt* pInterrupt);

  bool isBadHatch() const;

  enum point_face_flag
  {
    IN_FACE,
    OUTSIDE_FACE,
    ON_BOUNDARY,
    UNKNOWN
  };

  enum LOOP_TYPE
  {
    EXTERNAL,
    OUTERMOST,
    DEFAULT,
  };

  enum LOOP_FIND_METHOD
  {
    EXACT_IN,
    CLOSEST_IN
  };

  // Obtain the region contain the point
  point_face_flag getLoops(const OdGePoint2d &pnt, std::vector<OdGeVoidPointerArray> &boundary, std::vector<OdGeVoidPointerArray> &ids, 
    std::vector<LOOP_TYPE> &boundarTypes, LOOP_FIND_METHOD findMethod = EXACT_IN);

  // Get all the loops
  bool getLoops(std::vector<OdGeVoidPointerArray> &boundary, std::vector<OdGeVoidPointerArray> &ids, 
    std::vector<LOOP_TYPE> &boudaryTypes, std::vector<bool> &closedFlag);

  // get the face of this hatch
  int faceSize() const;

  // default loop : all the loops which are contained in the outermost loop of this face
  bool getSubFaces(int index, std::vector<OdGeVoidPointerArray> &boundary, std::vector<OdGeVoidPointerArray> &ids, 
    std::vector<LOOP_TYPE> &boudaryTypes, std::vector<bool> &closedFlag);

  point_face_flag pointOnFace(const OdGePoint2d &pnt, const OdGeVoidPointerArray &boundary);

  // find the topology relationship of the input loops
  bool getLoopTypes(const std::vector<OdGeVoidPointerArray> &allLoops, std::vector<LOOP_TYPE> &loopTypes);

  bool getGeoExtents(OdGeExtents2d &bBox) const;
private:
  OdGeHatchBuilderImpl *mImpl;
};
#endif